'use strict';

var main = main || {};

/***************** models  *****************/

var groups = null;

main.GroupList = function() {
  this.$this = $(this);
  this.groups = {};
};

main.GroupList.prototype.fetch = function() {
  var self = this;
  self.last_error = null;

  $.getJSON( "/api/v1/groups", function(data) {
    self.groups = [];
    for(var key in data.results) {
      self.groups[data.results[key].name] = new main.Group(data.results[key]);
    }
    self.$this.trigger('update');
  }).fail(function(jqxhr, textStatus, error) {
    if(error == "") {
      self.error("connection problem");
    } else {
      self.error(error);
    }
  });
};

main.GroupList.prototype.error = function(error) {
  this.last_error = error;
  this.$this.trigger('error');
};

main.Group = function(inst) {
  this.$this = $(this);
  this.name = inst.name;
  this.created = inst.created;
  this.updated = inst.updated;
  this.files = [];
  this.playlists = [];
};

main.Group.prototype.fetch_files = function() {
  var self = this;
  self.last_error = null;

  $.getJSON( "/api/v1/groups/" + self.name + "/files", function(data) {
    self.files = [];
    for(var key in data.results) {
      self.files.push(new main.File(data.results[key]));
    }
    self.$this.trigger('update-files');
  }).fail(function(jqxhr, textStatus, error) {
    if(error == "") {
      self.error("connection problem");
    } else {
      self.error(error);
    }
  });
};

main.Group.prototype.fetch_playlists = function() {
  var self = this;
  self.last_error = null;

  $.getJSON( "/api/v1/groups/" + self.name + "/playlists", function(data) {
    self.playlists = [];
    for(var key in data.results) {
      self.playlists.push(new main.Playlist(data.results[key]));
    }
    self.$this.trigger('update-playlists');
  }).fail(function(jqxhr, textStatus, error) {
    if(error == "") {
      self.error("connection problem");
    } else {
      self.error(error);
    }
  });
};

main.File = function(inst) {
  this.id = inst.id
  this.group = inst.group;
  this.created = inst.created;
  this.updated = inst.updated;
  this.source = inst.source;
  this.metadata = inst.metadata;
};

main.Playlist = function(inst) {
  this.id = inst.id
  this.group = inst.group;
  this.created = inst.created;
  this.updated = inst.updated;
  this.entries = inst.entries;
};

/***************** views  *****************/

var groupListView = null;

main.GroupListView = function(model) {
  this.model = model;

  var self = this;
  $(this.model).on('update', function() {
    self.render_list();
  });
  $(this.model).on('error', function() {
    self.render_error();
  });
};

main.GroupListView.prototype.render_list = function() {
  $('#error').hide();

  var list = $('#groupSelect');
  $('option', list).remove();
  list.append($('<option>', {value:'__none__', text:'choose...'}))

  for (var name in this.model.groups) {
    list.append($('<option>', {value:name, text:name}));
  }
};

main.GroupListView.prototype.render_error = function() {
  $('#error-message').text('Error fetching group listing: ' + this.model.last_error)
  $('#error').show();
};

var groupView = null;

main.GroupView = function(model) {
  this.model = model;
  this.$el = $('#hiddenTemplates .groupTemplate').clone().removeClass('groupTemplate');

  var self = this;
  var selectFiles = this.$el.find('#cardSelectFiles');
  var selectPlaylists = this.$el.find('#cardSelectPlaylists');
  var cardBody = self.$el.find('div.card-body');

  selectFiles.on('click', function() {
    selectFiles.addClass('active');
    selectPlaylists.removeClass('active');
    self.model.fetch_files();
    cardBody.text("...loading files...");
  });
  selectPlaylists.on('click', function() {
    selectPlaylists.addClass('active');
    selectFiles.removeClass('active');
    self.model.fetch_playlists();
    cardBody.text("...loading playlists...")
  });

  $(this.model).on('update-files', function() {
    self.render_files(cardBody);
  });
  $(this.model).on('update-playlists', function() {
    self.render_playlists(cardBody);
  });
  $(this.model).on('error', function() {
    self.render_error();
  });

  $('#groupView').html(this.$el);
};

main.GroupView.prototype.render_files = function(body) {
  $('#error').hide();

  var fileListView = new main.FileListView(this.model.files, this);
	fileListView.render();
  body.html(fileListView.$el);
};

main.GroupView.prototype.render_playlists = function(body) {
  $('#error').hide();
  body.html($('<pre>').text(JSON.stringify(this.model.playlists, null, 2)));
};

main.GroupView.prototype.render_error = function() {
  $('#error-message').text(this.model.last_error)
  $('#error').show();
};

main.FileListView = function(model) {
  this.model = model;
  this.$el = $('#hiddenTemplates .filesTemplate').clone().removeClass('filesTemplate');
};

main.FileListView.prototype.render = function() {
  var table = this.$el.find("tbody");
  for (var i = 0; i < this.model.length; i++) {
    var fileView = new main.FileView(this.model[i], this);
	  fileView.render();
    table.append(fileView.$el);
  }
};

main.FileView = function(model) {
  this.model = model;
  this.$el = $('#hiddenTemplates .fileTemplate tr').clone();
};

main.FileView.prototype.render = function() {
  this.$el.find('.fileId').text(this.model.id)
  this.$el.find('.fileArtist').text(this.model.metadata.artist)
  this.$el.find('.fileAlbum').text(this.model.metadata.album)
  this.$el.find('.fileTitle').text(this.model.metadata.title)
  this.$el.find('.fileSource').text(this.model.source.filename)
    // TODO: implement file delete
};

/***************** controller  *****************/

function main_init() {
  groups = new main.GroupList();
  groupListView = new main.GroupListView(groups);

  groups.fetch();
  $('#groupSelect').on('change', function(event) {
    var group_name = event.target.value;
    if(group_name != '__none__') {
      var group = groups.groups[group_name];
      groupView = new main.GroupView(group);
      group.fetch_files();
    } else {
      $('#groupView').empty();
    }
  });
}
